<?php
/*
 * Plugin Name: Linkodium CF7
 * Description: Интеграция Contact Form 7 и Linkodium CRM
 * Author: Linkodium
 * Author URI: https://linkodium.com
 * Version: 3.0
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}


define('QS_CF7_API_PLUGIN_PATH', plugin_dir_path(__FILE__));
define('QS_CF7_API_INCLUDES_PATH', plugin_dir_path(__FILE__) . 'includes/');
define('QS_CF7_API_TEMPLATE_PATH', get_template_directory());
define('QS_CF7_API_ADMIN_JS_URL', plugin_dir_url(__FILE__) . 'assets/js/');
define('QS_CF7_API_ADMIN_CSS_URL', plugin_dir_url(__FILE__) . 'assets/css/');
define('QS_CF7_API_FRONTEND_JS_URL', plugin_dir_url(__FILE__) . 'assets/js/');
define('QS_CF7_API_FRONTEND_CSS_URL', plugin_dir_url(__FILE__) . 'assets/css/');
define('QS_CF7_API_IMAGES_URL', plugin_dir_url(__FILE__) . 'assets/css/');

add_action('plugins_loaded', 'qs_cf7_textdomain');
/**
 * Load plugin textdomain.
 *
 * @since 1.0.0
 */
function qs_cf7_textdomain()
{
    load_plugin_textdomain('qs-cf7-api', false, basename(dirname(__FILE__)) . '/languages');
}

/**
 * The core plugin class
 */
require_once QS_CF7_API_INCLUDES_PATH . 'class.cf7-api.php';

/**
 * Activation and deactivation hooks
 *
 */
register_activation_hook(__FILE__, 'cf7_api_activation_handler');
register_deactivation_hook(__FILE__, 'cf7_api_deactivation_handler');


function cf7_api_activation_handler()
{
    do_action('cf7_api_activated');
}

function cf7_api_deactivation_handler()
{
    do_action('cf7_api_deactivated');
}

/**
 * Begins execution of the plugin.
 *
 * Init the plugin process
 *
 * @since    1.0.0
 */
function qs_init_cf7_api()
{
    global $qs_cf7_api;

    $qs_cf7_api = new QS_CF7_atp_integration();
    $qs_cf7_api->version = '1.1.1';
    $qs_cf7_api->plugin_basename = plugin_basename(__FILE__);

    $qs_cf7_api->init();

}

function qs_hook_session()
{
    require_once ABSPATH . 'wp-admin/includes/file.php';

    $homePath = get_home_path() . '/index.php';
    $sessionScript = "
    # LKDM CF7
    
    if (version_compare(PHP_VERSION, '7.0.0') >= 0) {
        if (function_exists('session_status') && session_status() == PHP_SESSION_NONE) {
            session_start(array(
                'cache_limiter' => 'private_no_expire',
            ));
        }
    } else if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
        if (function_exists('session_status') && session_status() == PHP_SESSION_NONE) {
            session_cache_limiter('private_no_expire');
            session_start();
        }
    } else {
        if (session_id() == '') {
            if (version_compare(PHP_VERSION, '4.0.0') >= 0) {
                session_cache_limiter('private_no_expire');
            }
            session_start();
        }
    }
    
    if (!isset(\$_SESSION['start_url']) && strpos(\$_SERVER['REQUEST_URI'], 'wp-json') === false) {
        \$_SESSION['start_url'] = (isset(\$_SERVER['HTTPS']) && \$_SERVER['HTTPS'] === 'on' ? \"https\" : \"http\") . \"://\" . \$_SERVER['HTTP_HOST'] . \$_SERVER['REQUEST_URI'];
        if (isset(\$_SERVER['HTTP_REFERER'])) \$_SESSION['referer'] = \$_SERVER['HTTP_REFERER']; else \$_SESSION['referer'] = \"\";
        \$_SESSION['user_agent'] = \$_SERVER['HTTP_USER_AGENT'];
        \$_SESSION['remote_ip'] = \$_SERVER['REMOTE_ADDR'];
    }
    # /LKDM CF7
    ";

    $indexContent = file_get_contents($homePath);
    $insertPosition = strrpos($indexContent, '<?php');

    if (strpos($indexContent, $sessionScript) === false) {
        file_put_contents($homePath, substr_replace($indexContent, $sessionScript, $insertPosition + 5, 0));
    }
}

if (function_exists('add_cacheaction') && isset($GLOBALS["super_cache_enabled"]) && $GLOBALS["super_cache_enabled"] && defined('WP_CACHE') && !defined('REST_REQUEST')) {
    add_cacheaction('add_cacheaction', function () {
        add_action('init', 'qs_hook_session');
    });
} else {
    add_action('init', 'qs_hook_session');
}

qs_init_cf7_api();
